import React, { Fragment, useEffect, useState } from 'react';
import { ModalEnum } from '../Mito';
import DefaultModal from '../DefaultModal';
/*
    A modal that allows a user to download their current sheet.

    It does this by:
    1. Getting a string representation of the sheet through the api
    2. Encoding that as a file
    3. Allowing the user to download that file

    To see more about this process, read documentation here:
    https://blog.logrocket.com/programmatic-file-downloads-in-the-browser-9a5186298d5c/
*/
const DownloadModal = (props) => {
    const [dataframeCSV, setDataframeCSV] = useState('');
    // We use a count to track the number of ...s to display.
    // 0 -> '', 1 -> '.', 2 -> '..', 3 -> '...'. Wraps % 4.
    const [indicatorState, setIndicatorState] = useState(0);
    const loadDataframeCSV = async () => {
        const loadedDataframeCSV = await props.mitoAPI.getDataframeAsCSV(props.selectedSheetIndex);
        setDataframeCSV(loadedDataframeCSV);
    };
    // Async load in the data from the mitoAPI
    useEffect(() => {
        void loadDataframeCSV();
    }, []);
    // Schedule a change to update the loading indicator, every .5 seconds
    useEffect(() => {
        const interval = setInterval(() => {
            setIndicatorState(indicatorState => indicatorState + 1);
        }, 500);
        return () => clearInterval(interval);
    }, []);
    const onDownload = () => {
        void props.mitoAPI.sendLogMessage('button_download_log_event', {
            sheet_index: props.selectedSheetIndex,
            df_name: props.dfName
        });
        props.setModal({ type: ModalEnum.None });
    };
    // Calculate the number of loading dots
    const someNumberOfDots = '.'.repeat(indicatorState % 4) + '.';
    return (React.createElement(DefaultModal, { header: 'Download Current Sheet', modalType: ModalEnum.Download, viewComponent: React.createElement(Fragment, null,
            dataframeCSV === '' &&
                React.createElement("div", { className: 'mt-2' },
                    React.createElement("p", { className: 'modal-loading-text' }, "Give us a few seconds to prepare your data for download."),
                    React.createElement("div", { className: 'modal-loading-dots mt-2' }, someNumberOfDots)),
            dataframeCSV !== '' &&
                React.createElement("p", null,
                    "Download ",
                    props.dfName,
                    " as a CSV.")), buttons: React.createElement(Fragment, null,
            React.createElement("div", { className: 'modal-close-button modal-dual-button-left', onClick: () => { props.setModal({ type: ModalEnum.None }); } }, " Close "),
            dataframeCSV === '' &&
                React.createElement("div", { className: 'modal-action-button modal-dual-button-right modal-button-disabled' }, "Download"),
            dataframeCSV !== '' &&
                React.createElement("a", { href: URL.createObjectURL(new Blob([dataframeCSV], { type: 'text/csv' })), download: props.dfName + '.csv', className: 'modal-action-button modal-dual-button-right', onClick: onDownload }, "Download")) }));
};
export default DownloadModal;
//# sourceMappingURL=DownloadModal.js.map