// Copyright (c) Mito
import React from 'react';
export var SortDirection;
(function (SortDirection) {
    SortDirection["ASCENDING"] = "ascending";
    SortDirection["DESCENDING"] = "descending";
    SortDirection["NONE"] = "none";
})(SortDirection || (SortDirection = {}));
/*
    A modal that allows a user to sort a column
*/
class SortCard extends React.Component {
    constructor(props) {
        super(props);
        this.setSortDirection = (sortDirection) => {
            // if the user toggled the button that was already selected, turn off the sort
            if (sortDirection == this.state.sortDirection) {
                this.setState({
                    sortDirection: SortDirection.NONE
                });
                return;
            }
            // otherwise the user selected a new direction, so set that as the sortDirection
            this.setState({
                sortDirection: sortDirection
            }, () => {
                // then sort the column
                void this.sendSortUpdateMessage();
            });
        };
        this.sendSortUpdateMessage = async () => {
            // sort the columns if the sortDirection is not None
            if (this.state.sortDirection != SortDirection.NONE) {
                const stepID = await this.props.mitoAPI.sendSortMessage(this.props.selectedSheetIndex, this.props.columnHeader, this.state.sortDirection, this.state.stepID);
                this.setState({ stepID: stepID });
            }
        };
        this.state = {
            sortDirection: SortDirection.NONE,
            stepID: ''
        };
        this.sendSortUpdateMessage = this.sendSortUpdateMessage.bind(this);
        this.setSortDirection = this.setSortDirection.bind(this);
    }
    render() {
        // determine css styling of sort buttons
        const ascendingButtonClass = this.state.sortDirection == SortDirection.ASCENDING ? 'sort-button sort-button-selected' : 'sort-button sort-button-unselected';
        const descendingButtonClass = this.state.sortDirection == SortDirection.DESCENDING ? 'sort-button sort-button-selected' : 'sort-button sort-button-unselected';
        return (React.createElement("div", null,
            React.createElement("div", { className: 'filter-modal-section-title' },
                React.createElement("p", { style: { alignSelf: 'center' } }, " Sort "),
                React.createElement("div", { className: 'sort-buttons-div' },
                    React.createElement("button", { className: ascendingButtonClass, onClick: () => this.setSortDirection(SortDirection.ASCENDING) }, "Ascending"),
                    React.createElement("button", { className: descendingButtonClass, onClick: () => this.setSortDirection(SortDirection.DESCENDING) }, "Descending")))));
    }
}
export default SortCard;
//# sourceMappingURL=SortCard.js.map