// Copyright (c) Mito
// Distributed under the terms of the Modified BSD License.
import React from 'react';
import '../../../../css/frustration-selection.css';
export var FrustrationRating;
(function (FrustrationRating) {
    FrustrationRating["Frustrating"] = "It was frustrating.";
    FrustrationRating["Ok"] = "It was ok.";
    FrustrationRating["Easy"] = "It was easy.";
})(FrustrationRating || (FrustrationRating = {}));
const FRUSTRATION_COLORS = {
    [FrustrationRating.Frustrating]: '#FF6666',
    [FrustrationRating.Ok]: '#FFC989',
    [FrustrationRating.Easy]: '#96D394',
};
/*
    An internal component that is just responsible for
    being an actual box that says a specific frustration
*/
function FrustrationSelectionBox(props) {
    return (React.createElement("div", { className: 'frustration-selection-box', style: {
            padding: props.padding || '',
            backgroundColor: FRUSTRATION_COLORS[props.frustrationRating],
            border: props.selectedFrustrationRating === props.frustrationRating ? '2px solid black' : '2px solid white'
        }, onClick: () => {
            // We let you toggle the selection on and off
            if (props.selectedFrustrationRating === props.frustrationRating) {
                props.setSelectedFrustrationRating(undefined);
            }
            else {
                props.setSelectedFrustrationRating(props.frustrationRating);
            }
        } },
        React.createElement("p", { className: 'm-auto' }, props.frustrationRating)));
}
/*
    Displays three boxes that the user can click
    rating an experience between frustrating and
    easy.
*/
function FrustrationSelection(props) {
    return (React.createElement("div", { className: 'frustration-selection-wrapper', style: {
            justifyContent: props.justifyContent || '',
        } },
        React.createElement(FrustrationSelectionBox, { frustrationRating: FrustrationRating.Frustrating, selectedFrustrationRating: props.selectedFrustrationRating, setSelectedFrustrationRating: props.setSelectedFrustrationRating, padding: props.padding }),
        React.createElement(FrustrationSelectionBox, { frustrationRating: FrustrationRating.Ok, selectedFrustrationRating: props.selectedFrustrationRating, setSelectedFrustrationRating: props.setSelectedFrustrationRating, padding: props.padding }),
        React.createElement(FrustrationSelectionBox, { frustrationRating: FrustrationRating.Easy, selectedFrustrationRating: props.selectedFrustrationRating, setSelectedFrustrationRating: props.setSelectedFrustrationRating, padding: props.padding })));
}
export default FrustrationSelection;
//# sourceMappingURL=FrustrationSelection.js.map