// Copyright (c) Mito
import React from 'react';
import { StepType } from '../../../types/StepTypes';
// Icons
import ImportIcon from '../../icons/ImportIcon';
import MergeIcon from '../../icons/MergeIcon';
import PivotIcon from '../../icons/PivotIcon';
import DeleteColumnIcon from '../../icons/DeleteColumnIcon';
import AddColumnIcon from '../../icons/AddColumnIcon';
import CustomDropdown from '../../elements/CustomDropdown';
import DropdownIcon from '../../icons/DropdownIcon';
import { FilterEmptyIcon } from '../../icons/FilterIcons';
import EditIcon from '../../icons/EditIcon';
import MitoIcon from '../../icons/MitoIcon';
/*
    Gets an icon for a specific step type, to display
    with that step
*/
export function getStepIcon(stepType) {
    switch (stepType) {
        case StepType.Initialize: return (React.createElement(MitoIcon, null));
        case StepType.AddColumn: return (React.createElement(AddColumnIcon, null));
        case StepType.DeleteColumn: return (React.createElement(DeleteColumnIcon, null));
        case StepType.RenameColumn: return (React.createElement(EditIcon, null));
        case StepType.ReorderColumn: return (React.createElement(EditIcon, null));
        case StepType.FilterColumn: return (React.createElement(FilterEmptyIcon, { height: '20', width: '20' }));
        case StepType.SetColumnFormula: return (React.createElement("div", { className: 'step-taskpane-missing-icon' }, "Fx"));
        case StepType.DataframeDelete: return (React.createElement(DeleteColumnIcon, null));
        case StepType.DataframeDuplicate: return (React.createElement(EditIcon, null));
        case StepType.DataframeRename: return (React.createElement(EditIcon, null));
        case StepType.SimpleImport: return (React.createElement(ImportIcon, null));
        case StepType.RawPythonImport: return (React.createElement(ImportIcon, null));
        case StepType.Sort: return (React.createElement(EditIcon, null));
        case StepType.Pivot: return (React.createElement(PivotIcon, { width: '20' }));
        case StepType.Merge: return (React.createElement(MergeIcon, { width: '27' }));
        default: return (React.createElement(EditIcon, null));
    }
}
/*
    An element in a list that displays information about a step, and
    eventually will allow the user to interact with that step (e.g.
    to start editing it).
*/
function StepDataElement(props) {
    const rollBackToStep = () => {
        props.setOpenedDropdown(false);
        void props.mitoAPI.checkoutStepByIndex(props.stepData.step_idx);
    };
    return (React.createElement("div", { className: 'step-taskpane-step-container' },
        React.createElement("div", { className: 'step-taskpane-step-container-left', style: { opacity: props.beforeCurrIdx ? '1' : '.5' }, onClick: rollBackToStep },
            React.createElement("div", { className: 'step-taskpane-step-icon' }, getStepIcon(props.stepData.step_type)),
            React.createElement("div", { className: 'step-taskpane-step-text' },
                React.createElement("div", { className: 'step-taskpane-step-header' }, props.stepData.step_display_name),
                React.createElement("div", { className: 'step-taskpane-step-subtext' }, props.stepData.step_description))),
        React.createElement("div", { className: 'step-taskpane-step-container-right', onClick: () => {
                props.setOpenedDropdown(!props.openedDropdown);
            } },
            React.createElement(DropdownIcon, null)),
        props.openedDropdown &&
            React.createElement(CustomDropdown, { closeDropdown: () => { props.setOpenedDropdown(false); } },
                React.createElement("div", { onClick: rollBackToStep }, "View this step"),
                React.createElement("div", { className: 'custom-dropdown-inactive' }, "Edit this step (coming soon)"))));
}
export default StepDataElement;
//# sourceMappingURL=StepDataElement.js.map