// Copyright (c) Mito
import { DOMWidgetModel, DOMWidgetView, } from '@jupyter-widgets/base';
import { MODULE_NAME, MODULE_VERSION } from './version';
// React
import React from 'react';
import ReactDOM from 'react-dom';
// Components
import Mito, { DataTypeInMito } from './components/Mito';
export class ExampleModel extends DOMWidgetModel {
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: ExampleModel.model_name, _model_module: ExampleModel.model_module, _model_module_version: ExampleModel.model_module_version, _view_name: ExampleModel.view_name, _view_module: ExampleModel.view_module, _view_module_version: ExampleModel.view_module_version, df_json: '' });
    }
}
ExampleModel.serializers = Object.assign({}, DOMWidgetModel.serializers);
ExampleModel.model_name = 'ExampleModel';
ExampleModel.model_module = MODULE_NAME;
ExampleModel.model_module_version = MODULE_VERSION;
ExampleModel.view_name = 'ExampleView'; // Set to null if no view
ExampleModel.view_module = MODULE_NAME; // Set to null if no view
ExampleModel.view_module_version = MODULE_VERSION;
import { ModalEnum } from "./components/Mito";
import { calculateAndSetHeaderHeight } from './utils/gridStyling';
import { MitoAPI, UserJsonFields } from './api';
import { TaskpaneType } from './components/taskpanes/taskpanes';
import { FeedbackOpenLocation } from './components/taskpanes/Feedback/FeedbackTaskpane';
export class ExampleView extends DOMWidgetView {
    /*
    We override the DOMWidgetView constructor, so that we can
    create a logging instance for this view.
  */
    initialize(parameters) {
        super.initialize(parameters);
    }
    /*
    We override the sending message utilities
  */
    send(msg) {
        var _a, _b;
        // If this is a message that makes a change (anything but a log_event or api_call or has_rendered_update),
        // then we make the system as loading, as we wait for a response
        if (msg['event'] !== 'log_event' && msg['event'] !== 'api_call' && msg['type'] !== 'has_rendered_update') {
            (_b = (_a = window.mitoMap) === null || _a === void 0 ? void 0 : _a.get(this.model.model_id)) === null || _b === void 0 ? void 0 : _b.setState({
                loading: true
            });
        }
        super.send(msg);
    }
    render() {
        var _a, _b;
        // Capture the send, to pass to the component
        const send = (msg) => {
            this.send(msg);
        };
        // TODO: there is a memory leak, in the case where
        // we rerender the component (e.g. we run the mito.sheet)
        // cell again. We need to clean up the component somehow!
        const model_id = this.model.model_id;
        const mitoAPI = new MitoAPI(send);
        ReactDOM.render(React.createElement(Mito, { mitoAPI: mitoAPI, dfNames: this.getDfNames(), sheetShapeArray: this.getSheetShapeArray(), currStepIdx: this.getCurrStepIdx(), sheetJSONArray: this.getSheetJSONArray(), columnSpreadsheetCodeJSONArray: this.getColumnSpreadsheetCodeJSONArray(), savedAnalysisNames: this.getSavedAnalysisNames(), columnFiltersArray: this.getColumnFilters(), columnTypeJSONArray: this.getColumnTypeJSONArray(), model_id: model_id, isLocalDeployment: this.getIsLocalDeployment(), userEmail: this.getUserEmail(), tourJSON: this.getTourJSON(), shouldUpgradeMitosheet: this.getShouldUpgradeMitosheet(), 
            /*
                If we should open the feedback taskpane, then we set it open from
                the start, so the user is prompted to give us feedback immediately
            */
            currOpenTaskpane: this.getShouldDisplayFeedback()
                ? { type: TaskpaneType.FEEDBACK, openLocation: FeedbackOpenLocation.AUTO_POPUP }
                : { type: TaskpaneType.NONE }, ref: (Mito) => {
                // The first time we render the item, register the Mito instance
                if (window.mitoMap === undefined) {
                    window.mitoMap = new Map();
                }
                window.mitoMap.set(model_id, Mito);
                // We also register the API for this Mito instance
                if (window.mitoAPIMap === undefined) {
                    window.mitoAPIMap = new Map();
                }
                window.mitoAPIMap.set(model_id, mitoAPI);
            }, stepDataList: this.getStepDataList() }), this.el);
        this.model.on('msg:custom', this.handleMessage, this);
        /*
        The first render of a widget happens when the mitosheet.sheet call is
        run. It does not occur when the page that this widget on is refreshed.
        
        We only need to send update events for df_names and reading an existing analysis
        in the case that this is the first render, and not when the page is refreshed.

        NOTE: Rerunning a mitosheet.sheet call that has an analysis in the cell
        below is _creating a new mitosheet_. Thus, this.getHasRendered() will
        return false in this case.
    */
        if (!this.getHasRendered()) {
            // Get the arguments passed to the mitosheet.sheet call
            (_a = window.commands) === null || _a === void 0 ? void 0 : _a.execute('get-args').then((args) => {
                this.send({
                    'event': 'update_event',
                    'type': 'args_update',
                    'args': args
                });
            });
            // Get any previous analysis and send it back to the model!
            (_b = window.commands) === null || _b === void 0 ? void 0 : _b.execute('read-existing-analysis').then(async (analysisName) => {
                // If there is no previous analysis, we just ignore this step
                if (!analysisName)
                    return;
                // We log that we read from below
                await mitoAPI.sendLogMessage('read_analysis_from_cell_below', {
                    analysis_name: analysisName
                });
                // And send it to the backend (we don't care when it terminates)
                await mitoAPI.sendUseExistingAnalysisUpdateMessage(analysisName, undefined, 
                /*
                    When we read in an analysis name from a cell, we replay this analysis
                    while also overwriting _everything_ that is already in the analysis.

                    This is to avoid issues w/ passing in a saved analysis to the mitosheet.sheet
                    call, where then rerunning the cell with this call w/ doubly-apply things.
                */
                true);
            });
        }
        // Let the backend know we have now rendered the sheet, and all future
        // renders therefor must be page refreshes, and so we let the backend 
        // know that we have rendered
        this.send({
            'event': 'update_event',
            'type': 'has_rendered_update'
        });
    }
    getHasRendered() {
        return this.model.get('has_rendered');
    }
    getSheetJSONArray() {
        const sheetJSONArray = [];
        try {
            const modelSheetJSONArray = JSON.parse(this.model.get('sheet_json'));
            sheetJSONArray.push(...modelSheetJSONArray);
        }
        catch (e) {
            // Suppress error
            console.error(e);
        }
        return sheetJSONArray;
    }
    getIsLocalDeployment() {
        return this.model.get('is_local_deployment');
    }
    getShouldDisplayFeedback() {
        return this.model.get('should_display_feedback');
    }
    getSheetShapeArray() {
        const sheetShapeArray = [];
        try {
            const sheetShapeJSONArrayUnparsed = JSON.parse(this.model.get('df_shape_json'));
            sheetShapeArray.push(...sheetShapeJSONArrayUnparsed);
        }
        catch (e) {
            // Suppress error
            console.error(e);
        }
        return sheetShapeArray;
    }
    getColumnTypeJSONArray() {
        const columnTypeJSONArray = [];
        try {
            const modelColumnTypeJSONArray = JSON.parse(this.model.get('column_type_json'));
            columnTypeJSONArray.push(...modelColumnTypeJSONArray);
        }
        catch (e) {
            // Suppress error
            console.error(e);
        }
        return columnTypeJSONArray;
    }
    getSavedAnalysisNames() {
        const savedAnalysisNames = [];
        try {
            const savedAnalysisNamesJSON = JSON.parse(this.model.get('saved_analysis_names_json'));
            savedAnalysisNames.push(...savedAnalysisNamesJSON);
        }
        catch (e) {
            // Suppress error
            console.error(e);
        }
        return savedAnalysisNames;
    }
    getCurrStepIdx() {
        return this.model.get('curr_step_idx');
    }
    getStepDataList() {
        const stepDataList = [];
        try {
            const savedStepDataList = JSON.parse(this.model.get('step_data_list_json'));
            stepDataList.push(...savedStepDataList);
        }
        catch (e) {
            // Suppress error
            console.error(e);
        }
        return stepDataList;
    }
    getColumnSpreadsheetCodeJSONArray() {
        return JSON.parse(this.model.get('column_spreadsheet_code_json'));
    }
    getDfNames() {
        const dfNames = [];
        const unparsedDfNames = this.model.get('df_names_json');
        try {
            dfNames.push(...JSON.parse(unparsedDfNames)['df_names']);
            // And then we extend it to the length of the number of sheets,
            // as the dfNames sometimes aren't pulled correctly!
            const modelSheetJSONArray = this.getSheetJSONArray();
            for (let i = dfNames.length; i < modelSheetJSONArray.length; i++) {
                dfNames.push(`df${i + 1}`);
            }
        }
        catch (e) {
            // Suppress error
            console.error(e);
        }
        return dfNames;
    }
    getCodeJSON() {
        const codeJSON = {
            imports: '# No imports',
            code: ['# No code has been written yet!', 'pass']
        };
        const unparsedCodeJSON = this.model.get('code_json');
        try {
            codeJSON['imports'] = JSON.parse(unparsedCodeJSON)['imports'];
            codeJSON['code'] = JSON.parse(unparsedCodeJSON)['code'];
        }
        catch (e) {
            // Suppress error
            console.error(e);
        }
        return codeJSON;
    }
    getAnalysisName() {
        return this.model.get('analysis_name');
    }
    getColumnFilters() {
        return JSON.parse(this.model.get('column_filters_json'));
    }
    getUserEmail() {
        return this.model.get(UserJsonFields.UJ_USER_EMAIL);
    }
    getShouldUpgradeMitosheet() {
        return this.model.get('should_upgrade_mitosheet');
    }
    getTourJSON() {
        const tourJSON = {
            typeOfDataInTool: DataTypeInMito.NONE,
            receivedTours: [],
            intendedBehavior: []
        };
        const typeOfDataInTool = this.model.get('data_type_in_mito');
        if (typeOfDataInTool === 'DataTypeInMito.NONE') {
            tourJSON['typeOfDataInTool'] = DataTypeInMito.NONE;
        }
        else if (typeOfDataInTool === 'DataTypeInMito.PROVIDED') {
            tourJSON['typeOfDataInTool'] = DataTypeInMito.PROVIDED;
        }
        else if (typeOfDataInTool === 'DataTypeInMito.TUTORIAL') {
            tourJSON['typeOfDataInTool'] = DataTypeInMito.TUTORIAL;
        }
        else if (typeOfDataInTool === 'DataTypeInMito.PERSONAL') {
            tourJSON['typeOfDataInTool'] = DataTypeInMito.PERSONAL;
        }
        else {
            tourJSON['typeOfDataInTool'] = DataTypeInMito.NONE;
        }
        tourJSON['receivedTours'] = JSON.parse(this.model.get('received_tours'));
        tourJSON['intendedBehavior'] = JSON.parse(this.model.get('intended_behavior'));
        if (tourJSON['receivedTours'] === null) {
            tourJSON['receivedTours'] = [];
        }
        if (tourJSON['intendedBehavior'] === null) {
            tourJSON['intendedBehavior'] = [];
        }
        return tourJSON;
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any, @typescript-eslint/explicit-module-boundary-types
    handleMessage(message) {
        /*
          This route handles the messages sent from the Python widget
        */
        var _a, _b, _c;
        const model_id = this.model.model_id;
        const mito = (_a = window.mitoMap) === null || _a === void 0 ? void 0 : _a.get(model_id);
        if (mito === undefined) {
            console.error("Error: a message was received for a mito instance that does not exist!");
            return;
        }
        console.log("Got a message, ", message);
        if (message.event === 'update_sheet') {
            console.log("Updating sheet.");
            mito.setState(() => {
                return {
                    sheetJSONArray: this.getSheetJSONArray(),
                    columnSpreadsheetCodeJSONArray: this.getColumnSpreadsheetCodeJSONArray(),
                    dfNames: this.getDfNames(),
                    sheetShapeArray: this.getSheetShapeArray(),
                    savedAnalysisNames: this.getSavedAnalysisNames(),
                    columnFiltersArray: this.getColumnFilters(),
                    columnTypeJSONArray: this.getColumnTypeJSONArray(),
                    loading: false,
                    stepDataList: this.getStepDataList(),
                    currStepIdx: this.getCurrStepIdx(),
                    tourJSON: this.getTourJSON()
                };
            }, () => {
                // Then recalculate the header height to make sure all headers are visible
                calculateAndSetHeaderHeight(model_id);
                // Then refresh the selected cell to make sure the formula did not get stuck in the cell renderer.
                mito.refreshSelectedCell();
            });
        }
        else if (message.event === 'update_code') {
            console.log('Updating code.');
            (_b = window.commands) === null || _b === void 0 ? void 0 : _b.execute('write-code-to-cell', {
                analysisName: this.getAnalysisName(),
                codeJSON: this.getCodeJSON()
            });
            mito.setState({
                loading: false
            });
        }
        else if (message.event === 'edit_error') {
            console.log("Updating edit error.");
            mito.setState({
                modalInfo: { type: ModalEnum.Error },
                errorJSON: message,
                loading: false
            });
        }
        else if (message.event === 'api_response') {
            // We get the MitoAPI associated with this Mito instance, and add to it's unconsumed queue
            const mitoAPI = (_c = window.mitoAPIMap) === null || _c === void 0 ? void 0 : _c.get(model_id);
            mitoAPI === null || mitoAPI === void 0 ? void 0 : mitoAPI.receiveResponse(message);
        }
    }
}
//# sourceMappingURL=widget.js.map